local common = require("mer.ashfall.common.common")
local versionController = require("mer.ashfall.common.versionController")
local config = require("mer.ashfall.config").config

local LINKS_LIST = {
    {
        text = " ",
        url = "https://github.com/jhaakma/ashfall/releases"
    },
    {
        text = "",
        url = "https://github.com/jhaakma/ashfall/wiki"
    },
    {
        text = "   Nexusmods",
        url = "https://www.nexusmods.com/morrowind/mods/49057"
    },
    {
        text = " ",
        url = "https://ko-fi.com/merlord"
    },
}
local CREDITS_LIST = {
    {
        text = ": Merlord",
        url = "https://www.nexusmods.com/users/3040468?tab=user+files",
    },
    {
        text = " : XeroFoxx",
        url = "https://www.youtube.com/channel/UCcx5oYt3NtLtadZTSjI3KEw",
    },
    {
        text = "  : Draconik",
        url = "https://www.nexusmods.com/morrowind/users/86600168",
    },
    {
        text = "   : Remiros",
        url = "https://www.nexusmods.com/morrowind/users/899234",
    },
    {
        text = "   : Vidi Aquam",
        url = "https://www.nexusmods.com/morrowind/mods/48782",
    },
    {
        text = "  (   ): Melchior Dahrk",
        url = "https://www.nexusmods.com/morrowind/users/962116"
    },
}
local SIDE_BAR_DEFAULT =
[[  ,      ,   .

    ,    .]]


local function addSideBar(component)
    local versionText = string.format("  %s", versionController.getVersion())
    component.sidebar:createCategory(versionText)
    component.sidebar:createInfo{ text = SIDE_BAR_DEFAULT}

    local linksCategory = component.sidebar:createCategory(":")
    for _, link in ipairs(LINKS_LIST) do
        linksCategory:createHyperLink{ text = link.text, url = link.url }
    end
    local creditsCategory = component.sidebar:createCategory(":")
    for _, credit in ipairs(CREDITS_LIST) do
        creditsCategory:createHyperLink{ text = credit.text, url = credit.url }
    end
end


local function registerModConfig()
    local template = mwse.mcm.createTemplate{ name = "", headerImagePath = "textures/ashfall/MCMHeader.tga" }
    template.onClose = function()
        config.save()
    end
    template:register()
    do --General Settings Page
        local pageGeneral = template:createSideBarPage({
            label = " ",
        })
        addSideBar(pageGeneral)

        do --Default Settings Category
            local categoryOverrides = pageGeneral:createCategory{
                label = " ",
                description = ",          ."
            }
            categoryOverrides:createYesNoButton{
                label = "  ",
                description = (
                    "      ,     , "..
                    "    ,     ,      " ..
                    "   ."
                ),
                variable = mwse.mcm.createTableVariable{
                    id = "overrideFood",
                    table = config,
                    restartRequired = true,
                    --restartRequiredMessage = "1Changing this setting requires a game restart to come into effect."
                }
            }

            categoryOverrides:createYesNoButton{
                label = "  ",
                description = (
                    "      .      "..
                    " ,       ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "overrideTimeScale", table = config },
                callback = function(self)
                    if tes3.player then
                        if self.variable.value == true then

                            local newTimeScale = config.manualTimeScale
                            tes3.setGlobal("TimeScale", newTimeScale)
                        end
                    end
                end
            }

            categoryOverrides:createSlider{
                label = "   ",
                description = ("     .  1       ; "
                .."       24 .  10      ,    . "
                .."(..     2,4 )  .. "
                .."\n\n    30 (1   = 48  ),      15-25."),
                min = 0,
                max = 50,
                step = 1,
                jump = 5,
                variable = mwse.mcm.createTableVariable{ id = "manualTimeScale", table = config },
                callback = function(self)
                    if tes3.player then
                        if config.overrideTimeScale == true then
                            tes3.setGlobal("TimeScale", self.variable.value)
                        end
                    end
                end
            }

        end

        do --Survival Mechanics Category
            local categorySurvival = pageGeneral:createCategory{
                label = " ",
                description = "\\   ."
            }
            categorySurvival:createYesNoButton{
                label = "",
                description = (
                    "   ,             . \n\n" ..
                    "      ,   ,  ,     ,  . \n\n" ..
                    "     ,       ,   ,   . \n\n" ..
                    " ,       ,     .\n\n"
                ),
                variable = mwse.mcm.createTableVariable{ id = "enableTemperatureEffects", table = config },
            }
            categorySurvival:createYesNoButton{
                label = "",
                description = (
                    "   ,     ,  . " ..
                    "    ,      ,    . "
                ),
                variable = mwse.mcm.createTableVariable{ id = "enableHunger", table = config },
            }
            categorySurvival:createYesNoButton{
                label = "",
                description = (
                    "   ,     ,   " ..
                    "      ,   .        ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "enableThirst", table = config },
                callback = tes3ui.updateInventoryTiles --to clear water bottle icons
            }
            categorySurvival:createYesNoButton{
                label = "",
                description = (
                    "   ,    ,      . " ..
                    "       \" \",              ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "enableTiredness", table = config },
            }
            categorySurvival:createYesNoButton{
                label = "",
                description = "   ,       .      .",
                variable = mwse.mcm.createTableVariable{ id = "enableBlight", table = config },
            }

        end --\Survival Mechanics Category

        do --Condition Updates Category
            local categoryConditions = pageGeneral:createCategory{
                label = "",
                description = "    ,    .",
            }

            categoryConditions:createOnOffButton{
                label = "",
                description = "      .",
                variable = mwse.mcm.createTableVariable{ id = "showTemp", table = config },
            }
            categoryConditions:createOnOffButton{
                label = "",
                description = "    .",
                variable = mwse.mcm.createTableVariable{ id = "showHunger", table = config },
            }
            categoryConditions:createOnOffButton{
                label = "",
                description = "     .",
                variable = mwse.mcm.createTableVariable{ id = "showThirst", table = config },
            }
            categoryConditions:createOnOffButton{
                label = "",
                description = "      .",
                variable = mwse.mcm.createTableVariable{ id = "showTiredness", table = config },
            }
            categoryConditions:createOnOffButton{
                label = "",
                description = "    .",
                variable = mwse.mcm.createTableVariable{ id = "showWetness", table = config },
            }
        end --\Condition Updates Category

        do -- Enable Features
            local categoryFeatures = pageGeneral:createCategory{
                label = "",
                description = "\\   ."
            }

            categoryFeatures:createYesNoButton{
                label = " ",
                description = "    ,         '  '.",
                variable = mwse.mcm.createTableVariable{ id = "bushcraftingEnabled", table = config }
            }

            categoryFeatures:createYesNoButton{
                label = "",
                description = "  .  ,      ,   .",
                variable = mwse.mcm.createTableVariable{ id = "enableSkinning", table = config }
            }

            categoryFeatures:createYesNoButton{
                label = "",
                description = "     ,     .             . ,       .",
                variable = mwse.mcm.createTableVariable{ id = "enableBranchPlacement", table = config }
            }

            categoryFeatures:createYesNoButton{
                label = "  ",
                description = (
                    "     NPC     . \n\n" ..
                    "     \"\". "
                ),
                variable = mwse.mcm.createTableVariable{ id = "showFrostBreath", table = config },
            }

            categoryFeatures:createYesNoButton{
                label = "  ",
                description = "   ,       ,        .  ,     .          ,        MGE XE,       .",
                variable = mwse.mcm.createTableVariable{ id = "disableHarvested", table = config}
            }

            categoryFeatures:createYesNoButton{
                label = "  ",
                description = (
                    "   , ,    ,     ,    ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "enableDiseasedMeat", table = config },
            }
        end

        do --Miscellanious Category

            local categoryMisc = pageGeneral:createCategory{
                label = "",
                description = "      .",
            }

            categoryMisc:createKeyBinder{
                label = " ",
                description = "     . ,        ,     (    ).  :  Shift.",
                allowCombinations = false,
                variable = mwse.mcm.createTableVariable{ id = "modifierHotKey", table = config },
            }

            categoryMisc:createYesNoButton{
                label = "  ",
                description = "   ,   .",
                variable = mwse.mcm.createTableVariable{ id = "showHints", table = config }
            }

            categoryMisc:createYesNoButton{
                label = "   ",
                description = "     ,      .",
                variable = mwse.mcm.createTableVariable{ id = "startingEquipment", table = config }
            }

            categoryMisc:createYesNoButton{
                label = "  \\",
                description = (
                    "   ,       .        1."
                ),
                variable = mwse.mcm.createTableVariable{ id = "needsCanKill", table = config },
            }

            categoryMisc:createYesNoButton{
                label = " ",
                description = "  ,        .",
                variable = mwse.mcm.createTableVariable{ id = "showBackpacks", table = config },
            }

            categoryMisc:createYesNoButton{
                label = " ",
                description = "   ,       ,    .",
                variable = mwse.mcm.createTableVariable{ id = "seeThroughTents", table = config }
            }

            categoryMisc:createYesNoButton{
                label = "    ",
                description = "   ,    (, ,   ) .          ,     - ,       .   ,    .   ,  ,  ,         .",
                variable = mwse.mcm.createTableVariable{ id = "atronachRecoverMagickaDrinking", table = config }
            }

            categoryMisc:createYesNoButton{
                label = " ",
                description = "   ,       .",
                variable = mwse.mcm.createTableVariable{ id = "potionsHydrate", table = config }
            }

            categoryMisc:createYesNoButton{
                label = "     ",
                description = (
                    "   ,     ,   ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "illegalHarvest", table = config },
            }

            categoryMisc:createYesNoButton{
                label = "    ",
                description = (
                    "   ,          ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "canCampInSettlements", table = config },
            }

            categoryMisc:createYesNoButton{
                label = "    ",
                description = (
                    "   ,            ."
                ),
                variable = mwse.mcm.createTableVariable{ id = "canRestOnGround", table = config },
            }

        end --\Miscellanious Category

    end -- \General Settings Page

    do --Mod values page
        local pageModValues = template:createSideBarPage{
            label = " "
        }
        addSideBar(pageModValues)

        do -- updates
            local categoryUpdates = pageModValues:createCategory{
                label = " ",
                description = "   .",
            }

            categoryUpdates:createSlider{
                label = "   ()",
                description = "     .      ,       . .   .      ,   .",
                min = 10,
                max = 2000,
                step = 10,
                jump = 100,
                variable = mwse.mcm.createTableVariable{ id = "rayTestUpdateMilliseconds", table = config },
            }
        end

        do -- Temperature
            local categoryTemperature = pageModValues:createCategory{
                label = "",
                description = " .",
            }
            categoryTemperature:createSlider{
                label = " : %s%%",
                description = string.format("    ."
                    .. "\n\n : %s.",
                    common.defaultValues.globalColdEffect
                ),
                min = -50,
                max = 50,
                step = 1,
                variable = mwse.mcm.createTableVariable{ id = "globalColdEffect", table = config },
                callback = function()
                    common.data.globalColdEffect = 1 + config.globalColdEffect * 0.01
                end
            }
            categoryTemperature:createSlider{
                label = " : %s%%",
                description = string.format("    ."
                    .. "\n\n : %s.",
                    common.defaultValues.globalWarmEffect
                ),
                min = -50,
                max = 50,
                step = 1,
                variable = mwse.mcm.createTableVariable{ id = "globalWarmEffect", table = config },
                callback = function()
                    common.data.globalWarmEffect = 1 + config.globalWarmEffect * 0.01
                end
            }
        end
        do --Hunger Category
            local categoryTime = pageModValues:createCategory{
                label = "",
                description = "  .",
            }


            categoryTime:createSlider{
                label = " ",
                description = string.format(
                    ",      .   10,    1%%   "
                    .."(   ). "
                    .."\n\n   : %s.",
                    common.defaultValues.hungerRate
                ),
                min = 0,
                max = 100,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "hungerRate", table = config },
            }
        end --\Hunger category

        do --Thirst Category
            local categoryThirst = pageModValues:createCategory{
                label = "",
                description = "  .",
            }

            categoryThirst:createSlider{
                label = " ",
                description = string.format(
                    ",     .   10,    1%%   "
                    .."(   ). "
                    .."\n\n   : %s.",
                    common.defaultValues.thirstRate
                ),
                min = 0,
                max = 100,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "thirstRate", table = config },
            }
        end--\Thirst Category

        do --Sleep Category
            local categorySleep = pageModValues:createCategory{
                label = "",
                description =  " .",
            }

            categorySleep:createSlider{
                label = " ",
                description = string.format(
                    ",      .   10    1%%  . "
                    .."\n\n   : %s.",
                    common.defaultValues.loseSleepRate
                ),
                min = 0,
                max = 200,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "loseSleepRate", table = config },
            }
            categorySleep:createSlider{
                label =  "  ( )",
                description = string.format(
                    ",         .   10    1%%  . "
                    .."\n\n  ( )  : %s.",
                    common.defaultValues.loseSleepWaiting
                ),
                min = 0,
                max = 200,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "loseSleepWaiting", table = config },
            }

            categorySleep:createSlider{
                label = "  ( )",
                description = string.format(
                    ",     ,   . "
                    .."  10,    1%%  . "
                    .."\n\n  ( ): %s.",
                    common.defaultValues.gainSleepRate
                ),
                min = 0,
                max = 200,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "gainSleepRate", table = config },
            }
            categorySleep:createSlider{
                label = "  ( )",
                description = string.format(
                    ",    ,     . "
                    .."  10,    1%%  . "
                    .."\n\n  ( ): %s.",
                    common.defaultValues.gainSleepBed
                ),
                min = 0,
                max = 200,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "gainSleepBed", table = config },
            }
        end --\Sleep Category

        do --Natural Materials Placement
            local categoryNatualMaterials = pageModValues:createCategory{
                label = " ",
                description = "  .",
            }

            --Determines frequence of materials such as wood, stone, flint etc that spawn in the world
            categoryNatualMaterials:createSlider{
                label = "  ",
                description = string.format(
                    ",    ,  ,   ,   . "
                    .."\n\n    : %s.",
                    common.defaultValues.naturalMaterialsMultiplier
                ),
                min = 0,
                max = 100,
                step = 1,
                jump = 10,
                variable = mwse.mcm.createTableVariable{ id = "naturalMaterialsMultiplier", table = config },
            }
        end
    end --\mod values page

    do --Exclusions Page
        template:createExclusionsPage{
            label = "  \\",
            description = (
                ",              .         ,       ."
            ),
            variable = mwse.mcm.createTableVariable{ id = "blocked", table = config },
            filters = {
                {
                    label = "",
                    type = "Plugin",
                },
                {
                    label = "",
                    type = "Object",
                    objectType = tes3.objectType.ingredient,
                },
                {
                    label = "",
                    type = "Object",
                    objectType = tes3.objectType.alchemy
                }
            }
        }

    end --\Exclusions Page

    local function offersService(npcObject, service)
        if npcObject.class and npcObject.class[service] then
            return true
        end
        if npcObject.aiConfig and npcObject.aiConfig[service] then
            return true
        end
        return false
    end

    do --Camping gear merchants
        template:createExclusionsPage{
            label = " ",
            description = "    ,      .       ,        ,   .  ,          ,     ,    .",
            variable = mwse.mcm.createTableVariable{ id = "campingMerchants", table = config },
            leftListLabel = "  ",
            rightListLabel = "",
            filters = {
                {
                    label = "Merchants",
                    callback = function()
                        --Check if npc is able to sell any guar gear
                        local function canSellGear(obj)
                            local bartersFields = {
                                "bartersMiscItems",
                            }
                            for _, field in ipairs(bartersFields) do
                                if offersService(obj, field) then
                                    return true
                                end
                            end
                            return false
                        end

                        local merchants = {}
                        ---@param obj tes3npcInstance
                        for obj in tes3.iterateObjects(tes3.objectType.npc) do
                            if not (obj.baseObject and obj.baseObject.id ~= obj.id ) then
                                if canSellGear(obj) then
                                    merchants[#merchants+1] = (obj.baseObject or obj).id:lower()
                                end
                            end
                        end
                        table.sort(merchants)
                        return merchants
                    end
                }
            }
        }
    end

    do --Camping gear merchants
        template:createExclusionsPage{
            label = " \\",
            description = "    ,            .       ,        ,   .  ,          ,     ,    .",
            variable = mwse.mcm.createTableVariable{ id = "foodWaterMerchants", table = config },
            leftListLabel = "  \\",
            rightListLabel = "",
            filters = {
                {
                    label = "Merchants",
                    callback = function()
                        --Check if npc is able to sell any guar gear
                        local function canSellGear(obj)
                            if obj.class then
                                local bartersFields = {
                                    "bartersAlchemy",
                                    "bartersIngredients"
                                }
                                for _, field in ipairs(bartersFields) do
                                    if offersService(obj, field) then
                                        return true
                                    end
                                end
                            end
                            return false
                        end

                        local merchants = {}
                        ---@param obj tes3npcInstance
                        for obj in tes3.iterateObjects(tes3.objectType.npc) do
                            if not (obj.baseObject and obj.baseObject.id ~= obj.id ) then
                                if canSellGear(obj) then
                                    merchants[#merchants+1] = (obj.baseObject or obj).id:lower()
                                end
                            end
                        end
                        table.sort(merchants)
                        return merchants
                    end
                }
            }
        }
    end

    do --Dev Options
        local pageDevOptions = template:createSideBarPage{
            label = "  ",
            description = "    ..  ,   ,  .",
        }

        pageDevOptions:createOnOffButton{
            label = " ",
            description = "   ,         .",
            variable = mwse.mcm.createTableVariable{ id = "checkForUpdates", table = config },
            restartRequired = true,
        }

        pageDevOptions:createYesNoButton{
            label = "    ",
            description = "         .   ,        .",
            variable = mwse.mcm.createTableVariable{ id = "doIntro", table = config }
        }

        pageDevOptions:createOnOffButton{
            label = " ",
            description = "   .   .",
            variable = mwse.mcm.createTableVariable{ id = "debugMode", table = config }
        }

        pageDevOptions:createDropdown{
            label = "Log Level",
            description = "     mwse.log.  INFO,     .",
            options = {
                { label = "TRACE", value = "TRACE"},
                { label = "DEBUG", value = "DEBUG"},
                { label = "INFO", value = "INFO"},
                { label = "ERROR", value = "ERROR"},
                { label = "NONE", value = "NONE"},
            },
            variable = mwse.mcm.createTableVariable{ id = "logLevel", table = config },
            callback = function(self)
                for _, log in ipairs(common.loggers) do
                    log:setLogLevel(self.variable.value)
                end
            end
        }

        pageDevOptions:createButton{
            buttonText = "   ",
            description = "      Morrowind/MWSE.log.       ,    ,   ,     MWSE.log Merlord   Morrowind Modding  Discord.",
            callback = function()
                mwse.log("Ashfall Data:")
                mwse.log(json.encode(common.data, { indent = true }))
            end,
            inGameOnly = true
        }
    end --\Dev Options
end

event.register("modConfigReady", registerModConfig)