local Activator = require("mer.ashfall.activators.Activator")
local this = {}

---@class Ashfall.Activator.Config
---@field name? string The name of the activator
---@field type string The type of the activator
---@field mcmSetting? string The name of the MCM setting that controls whether this activator is active
---@field ids? table<string, table> A table of ids to register for this activator. Key: id, Value: table of data
---@field patterns? table<string, boolean> A table of patterns to register for this activator. Key: pattern, Value: true
---@field isStewer boolean Whether this activator is a stewer
---@field owned? boolean Whether this activator is owned by NPCs, such as kegs


this.types = {
    waterSource = "waterSource",
    dirtyWaterSource = "dirtyWaterSource",
    cookingUtensil = "cookingUtensil",
    fire = "fire",
    campfire = "campfire",
    woodSource = "woodSource",
    stump = "stump",
    resinSource = "resinSource",
    vegetation = "vegetation",
    branch = "branch",
    cauldron = "cauldron",
    cushion = "cushion",
    hearth = "hearth",
    stove = "stove",
    partial = "partial",
    planter = "planter",
    teaWarmer = "teaWarmer",
    kettle = "kettle",
    cookingPot = "cookingPot",
    waterContainer = "waterContainer",
    stoneSource = "stoneSource",
    waterFilter = "waterFilter",
    woodStack = "woodStack",
    moonStone = "moonStone",
}

this.subTypes = {
    partial = "partial",
    waterDirty = "waterDirty",
    water = "water",
    waterClean = "waterClean",
    basin = "basin",
    waterJug = "waterJug",
    well = "well",
    keg = "keg",
    vegetation = "vegetation",
    tree = "tree",
    stump = "stump",
    deadTree = "deadTree",
    wood = "wood",
    fire = "fire",
    campfire = "campfire",
    hearth = "hearth",
    fireplace = "fireplace",
    stove = "stove",
    cushion = "cushion",
    cauldron = "cauldron",
    teaWarmer = "teaWarmer",
    kettle = "kettle",
    cookingPot = "cookingPot",
    waterContainer = "waterContainer",
    stoneSource = "stoneSource",
    waterFilter = "waterFilter",
    woodStack = "woodStack",
}

this.list = {}

--[[
    The partial activator is any activator that
    only shows a tooltip and has functionality
    when looking at specific NiNodes.

    There's nothing special going on behind the scenes,
    it's literally just registering an activator with no name
    and no functionality.
]]
this.list.partial = Activator:new({
    name = nil,
    type = this.types.partial,
    ids = {}
})

this.list.waterDirty = Activator:new{
    name = " ()",
    type = this.types.dirtyWaterSource,
    mcmSetting = nil,
    ids = {}
}
this.list.water = this.list.waterDirty
this.list.waterClean = Activator:new{
    name = " ()",
    type = this.types.waterSource,
    mcmSetting = nil,
    ids = {}
}
this.list.basin = Activator:new{
    name = "",
    type = this.types.waterSource,
    mcmSetting = nil,
    ids = {}
}
this.list.waterJug = Activator:new{
    name = "  ",
    type = this.types.waterSource,
    mcmSetting = nil,
    ids = {}
}
this.list.well = Activator:new{
    name = "",
    type = this.types.waterSource,
    mcmSetting = nil,
    ids = {}
}
this.list.keg = Activator:new{
    name = "",
    type = this.types.waterSource,
    mcmSetting = nil,
    ids = {},
    owned = true,
}
this.list.vegetation = Activator:new{
    name = "",
    type = this.types.vegetation,
    mcmSetting = "bushcraftingEnabled",
    patterns = {
    },
    ids = {}
}

this.list.tree = Activator:new{
    name = "",
    type = this.types.resinSource,
    mcmSetting = nil,
    ids = {},
    patterns = {
        ["vurt_baobab"] = true,
        ["vurt_bctree"] = true,
        ["vurt_bentpalm"] = true,
        ["vurt_decstree"] = true,
        ["vurt_neentree"] = true,
        ["vurt_palm"] = true,
        ["vurt_unicy"] = true,
        ["pine_tree"] = true,--vsw
        ["mr_flora_graze_tree"] = true, --Rebirth
    },
}
this.list.stump = Activator:new{
    name = "",
    type = this.types.stump,
    mcmSetting = nil,
    ids = {},
}
this.list.deadTree = Activator:new{
    name = " ",
    type = this.types.woodSource,
    mcmSetting = nil,
    ids = {},
}
this.list.wood = Activator:new{
    name = "",
    type = this.types.woodSource,
    mcmSetting = nil,
    ids = {},
    patterns = {}
}
this.list.fire = Activator:new{
    name = "",
    type = this.types.fire,
    patterns = {}
}

this.list.campfire = Activator:new{
    name = "",
    type = this.types.campfire,
    mcmSetting = nil,
    ids = {},
}

this.list.hearth = Activator:new{
    name = "",
    type = this.types.campfire,
    mcmSetting = nil,
    ids = {}
}

this.list.fireplace = Activator:new{
    name = "",
    type = this.types.campfire,
    mcmSetting = nil,
    ids = {}
}
--Stove
this.list.stove = Activator:new{
    name = "",
    type = this.types.campfire,
    mcmSetting = nil,
    ids = {}
}

this.list.cushion = Activator:new{
    name = "",
    type = this.types.cushion,
    mcmSetting = nil,
    ids = {
        furn_de_cushion_round_01 = { height = 20 },
        furn_de_cushion_round_02 = { height = 20 },
        furn_de_cushion_round_03 = { height = 20 },
        furn_de_cushion_round_04 = { height = 20 },
        furn_de_cushion_round_05 = { height = 20 },
        furn_de_cushion_round_06 = { height = 20 },
        furn_de_cushion_round_07 = { height = 20 },
        furn_de_cushion_square_01 = { height = 10 },
        furn_de_cushion_square_02 = { height = 10 },
        furn_de_cushion_square_03 = { height = 10 },
        furn_de_cushion_square_04 = { height = 10 },
        furn_de_cushion_square_05 = { height = 10 },
        furn_de_cushion_square_06 = { height = 10 },
        furn_de_cushion_square_07 = { height = 10 },
        furn_de_cushion_square_08 = { height = 10 },
        furn_de_cushion_square_09 = { height = 10 },
        ss20_dae_cushion_round_01 = { height = 20 },
        ss20_dae_cushion_round_02 = { height = 20 },
        ss20_dae_cushion_round_03 = { height = 20 },
        ss20_dae_cushion_round_04 = { height = 20 },
        ss20_dae_cushion_round_05 = { height = 20 },
        ss20_dae_cushion_square_01 = { height = 10 },
        ss20_dae_cushion_square_02 = { height = 10 },
        ss20_dae_cushion_square_03 = { height = 10 },
        ss20_dae_cushion_square_04 = { height = 10 },
        ss20_dae_cushion_square_05 = { height = 10 },
    },
}

this.list.cauldron = Activator:new{
    name = "",
    type = this.types.cauldron,
    mcmSetting = nil,
    ids = {},
    isStewer = true
}

this.list.teaWarmer = Activator:new{
    type = this.types.teaWarmer,
    mcmSetting = nil,
    ids = {},
}

this.list.kettle = Activator:new{
    type = this.types.kettle,
    mcmSetting = nil,
    ids = {},
}

this.list.cookingPot = Activator:new{
    type = this.types.cookingPot,
    mcmSetting = nil,
    ids = {}
}

this.list.waterContainer = Activator:new{
    type = this.types.waterContainer,
    mcmSetting = nil,
    ids = {}
}

this.list.stoneSource = Activator:new{
    name = "",
    type = this.types.stoneSource,
    mcmSetting = nil,
    patterns = {
        ["terrain_rock"] = true,
        ["terrain_ashland_rock"] = true,
    }
}

this.list.waterFilter = Activator:new{
    name = " ",
    type = this.types.waterFilter,
    ids = {}
}

this.list.woodStack = Activator:new{
    name = " ",
    type = this.types.woodStack,
    ids = { },
}

this.list.moonStone = Activator:new{
    name = " ",
    type = this.types.moonStone,
    ids = {
        ggw_rock_moonstone01  = true
    }
}

return this