local common = require ("mer.ashfall.common.common")
local WoodStack = require("mer.ashfall.items.woodStack")
return {
    dropText = function(targetRef, item, itemData)
        return " "
    end,
    ---@param targetRef tes3reference
    canDrop = function(targetRef, item, _itemData)

        local hasNodes = targetRef.sceneNode:getObjectByName("DROP_WOODSTACK")
            and targetRef.sceneNode:getObjectByName("SWITCH_WOODSTACK")
        if not hasNodes then
            return false
        end

        local id = item.id:lower()
        local isFirewood = id == common.staticConfigs.objectIds.firewood

        if not isFirewood then
            return false
        end
        local hasRoom = (not targetRef.data.woodAmount)
            or ( targetRef.data.woodAmount < WoodStack.getCapacity(targetRef.object.id) )
        if not hasRoom then
            return false, string.format("  ")
        end
        return true
    end,
    onDrop = function(targetRef, reference)

        local stackCount = common.helper.getStackCount(reference)

        targetRef.data.woodAmount = targetRef.data.woodAmount or 0
        local currentCapacity = WoodStack.getCapacity(targetRef.object.id) - targetRef.data.woodAmount

        local woodAdded = math.min(stackCount,currentCapacity)
        local woodRemaining = stackCount - woodAdded

        targetRef.data.woodAmount = targetRef.data.woodAmount + woodAdded
        if woodRemaining == 0 then
            reference:delete()
            tes3.playSound{ reference = tes3.player, sound = "ashfall_add_wood"  }
        else
            reference.attachments.variables.count = reference.attachments.variables.count - woodAdded
            common.helper.pickUp(reference)
            tes3.playSound{ reference = tes3.player, sound = "ashfall_add_wood"  }
        end
        tes3.messageBox(": %s%s.", woodAdded > 1 and woodAdded .. " " or "", reference.object.name)
        event.trigger("Ashfall:UpdateAttachNodes", { reference = targetRef})
    end
}